@@ -24,7 +24,7 @@
 #include "at_view.h"
 #include "at_type.h"
 
-#define MIX_CHUNK_SIZE 1024
+#define MIX_CHUNK_SIZE 512
 
 /* static variables */
 GSList *clip_list = NULL;
@@ -405,7 +405,16 @@ at_clip_update (Clip *clip)
     at_clip_mix_tracks (clip, 0, clip->length);
     
 } 
-    
+
+/* called from gtk_timeout_add ()'s */
+gint 
+at_clip_delayed_update (gpointer data)
+{
+  Clip *clip = data;
+  at_clip_update (clip);
+
+  return (FALSE);
+}
 
 
 void
@@ -424,9 +433,13 @@ at_clip_new_cmd_callback (GtkWidget *widget, gpointer data)
 		     at_clip_get_track_index (view->clip,
 					      view->clip->active_track));
 
+  /* delete the region created with the track */
+  region_delete (track->data);
+  /* set it to the composite data */
   track->data = clip->composite;
   clip->composite_track = track;
-
+  track->is_composite = TRUE;
+  
   /* create a first track for recording on */
   track = at_track_new (view->clip->ID, 0, 200, AT_RL, "New Track", 1.0, 0);
   at_clip_add_track (view->clip, track, 
@@ -459,25 +472,39 @@ at_clip_mix_tracks (Clip *clip, size_t offset, size_t length)
   gint track_count = 0;
   TmpRegion *mix_dest;
   Mix *mix_data = &clip->mix;
+  gint mix_needed = FALSE;
 
-  if (!clip->dirty)
+  /* check for dirty tracks to see if we require remixing */
+  while (list) {
+    track = (Track *) list->data;
+    if (track->dirty)
+      mix_needed = TRUE;
+    list = list->next;
+  }
+
+  if (!mix_needed && !clip->dirty)
     return;
 
-  clip->dirty = FALSE;
-
   g_print ("mixing tracks.... \n");
 
+  /* if we're already in the process of mixing, kill it,
+     and restart. */
+  if (clip->mixing)
+    at_clip_stop_idle_mix(clip);
+
   region_resize (clip->composite, 0, clip->length);
   /* zero out the composition track */
   mix_dest = tmp_region_new (clip->composite, 0, clip->length);
   tmp_region_zero (mix_dest);
 
   /* Create a list of all tracks that need mixing */
+
+  list = clip->tracks;
   while (list)
     {
       track = (Track *) list->data;
       
-      if (!track->mute)
+      if (!track->mute && !track->is_composite)
 	{
 	  track_list = g_slist_prepend (track_list, track);
 	  track_count ++;
@@ -504,18 +531,19 @@ at_clip_mix_tracks (Clip *clip, size_t offset, size_t length)
   
 }
 
+/* this is going to get big.... */
+
 static gint
 at_clip_idle_mix (gpointer data)
 {
   Clip *clip = data;
   Mix *mix = &clip->mix;
+  Track *track;
   gint track_done = FALSE;
   gint count, i;
   atdata *dest_data, *src_data;
   gint track_size;
 
-
-
   /* check if there's a track on the queue */
   if (!mix->current_track) {
 
@@ -523,17 +551,12 @@ at_clip_idle_mix (gpointer data)
        we're done mixing */
     if (mix->list == NULL) {
       /* we have now completed the mixing */
-      /* divide data by number of tracks */
       dest_data = mix->composite_region->data;
-      track_size = mix->composite_region->bps * mix->composite_region->length / sizeof (atdata);
+      track_size = mix->composite_region->bps * mix->composite_region->length / 
+	sizeof (atdata);
       
-      if (mix->track_count != 1) {
-	for (i=0; i < track_size; i++)
-	  dest_data[i] /= mix->track_count;
-      }
-
       g_print ("Idle mixing completed... \n");
-
+      
       g_slist_free (mix->track_list);
       tmp_region_destroy (mix->composite_region);
       
@@ -548,15 +571,19 @@ at_clip_idle_mix (gpointer data)
     /* get next track from linked list */
     mix->current_track = mix->list->data;
 
+    track = mix->current_track;
+    
     /* map track */
-    mix->track_region = tmp_region_new (mix->current_track->data, 0,
-					mix->current_track->length);
+    mix->track_region = tmp_region_new (track->data, 0,
+					track->length);
     /* since this is a new track, set the position to 0 */
     mix->position = 0;
     mix->list = mix->list->next;
   }
+    
+  track = mix->current_track;
 
-  track_size = mix->current_track->data->bps * mix->current_track->length / sizeof (atdata);
+  track_size = track->data->bps * track->length / sizeof (atdata);
 
   count = MIX_CHUNK_SIZE * mix->bps;
 
@@ -575,24 +602,51 @@ at_clip_idle_mix (gpointer data)
   /* perform the actual mixing */
   /* FIXME: handle number of channels correctly */
   for (i=0; i < count; i++) {
-      dest_data[i] += src_data[i] * mix->current_track->volume;
+      dest_data[i] += (src_data[i] * track->volume);
   }
-  
+
+
   if (track_done) {
     g_print ("track completed\n");
     tmp_region_destroy (mix->track_region);
+    track->dirty = FALSE;
     mix->current_track = NULL;
   }
 
   return (TRUE);
 }
 
+void
+at_clip_stop_idle_mix(Clip *clip)
+{
+  Mix *mix;
+  mix = &clip->mix;
+
+  if (clip->mixing == FALSE)
+    return;
+  
+  clip->mixing = FALSE;
+
+  /* since mixing was not complete, we remark as dirty */
+  clip->dirty = TRUE;
+
+
+  if (mix->composite_region)
+    tmp_region_destroy (mix->composite_region);
+  
+  g_slist_free (mix->track_list);
+  gtk_idle_remove (mix->record_tag);
+}
+
 
 static void 
 at_clip_mixing_complete (Clip *clip)
 {
   clip->composite_track->length = clip->composite->length;
   at_track_build_preview (clip->composite_track);
+  at_track_adjust_volume_of_preview (clip->composite_track, 1.0, 
+				     clip->composite_track->volume); 
+  at_track_update_preview (clip->composite_track);
   clip->mixing = FALSE;
 }
 
@@ -606,3 +660,6 @@ at_clip_mixing_complete (Clip *clip)
 
 
 
+
+
+
