@@ -49,7 +49,7 @@ struct _Clip
   gint undo_on;               /* is undo enabled?         */
   
   Mix mix;                    /* mixing information for idle mixing */
-  gint mixing;
+  gint mixing;                /* are we currently mixing ? */
     
   gint instance_count;        /* number of instances      */
   gint ref_count;             /* number of references     */
@@ -88,7 +88,7 @@ Clip *            at_clip_get_named            (char *);
 Clip *            at_clip_get_ID               (int);
 void              at_clip_resize               (Clip *, size_t, size_t);
 
-void              at_clip_mix_invalidate (Clip *);
+void              at_clip_mix_invalidate       (Clip *);
 void              at_clip_set_active_track     (Clip *, gint);
 gint              at_clip_get_track_index      (Clip *, gint);
 
@@ -104,6 +104,9 @@ void              at_clip_play_start           (Clip *clip);
 gint              at_clip_playing              (Clip *clip);
 
 void              at_clip_update               (Clip *clip);
+gint              at_clip_delayed_update       (gpointer data);
+void              at_clip_stop_idle_mix        (Clip *clip);
+
 
 void              at_clip_new_cmd_callback     (GtkWidget *widget, gpointer data);
 
