@@ -106,11 +106,14 @@ void at_com_play_start (GtkWidget *widget, gpointer data)
 {
     View *view = (View *) data;
     Clip *clip = view->clip;
-
+    
     if (at_clip_playing(clip))
       return;
-    at_ipc_signal_emit (play_module, "start_playing (string)",
-			at_clip_get_play_file(clip));
+    at_ipc_signal_emit (play_module, "start_playing (string, float)",
+			at_clip_get_play_file(clip), 
+			clip->composite_track->volume);
+
+    g_print ("clip->composite_track->volume is %f\n", clip->composite_track->volume);
 
     /* keep track of playing */
     at_clip_play_start (clip);
