@@ -1,6 +1,4 @@
 #include <gtk/gtk.h>
-#include <string.h>
-#include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -9,6 +7,7 @@
 #include "at_type.h"
 #include "at_track.h"
 #include "at_view.h"
+#include "at_delay.h"
 #include "libatech.h"
 
 /* I don't like this.. it needs to be in the clip structure and the clip
@@ -63,6 +62,10 @@ at_track_new (gint clip_ID,
   new_track->preview[1] = NULL;
   new_track->preview_len = 0;
   new_track->preview_compression = 800;
+  new_track->preview_dirty = FALSE;
+
+  new_track->dirty = FALSE;
+  new_track->is_composite = FALSE;
 
   track_list = g_slist_append (track_list, new_track);
 
@@ -81,7 +84,7 @@ at_track_delete (Track *track)
       adjustment_delete (track->adjustment);
     }
     */
-
+  
   track_list = g_slist_remove (track_list, track);
   g_free (track);
 }
@@ -112,7 +115,6 @@ at_track_translate (Track *track, size_t new_offset)
   
   at_track_dirty (track);
   at_track_invalidate_thumbnail (track->ID);
-
 }
 
 void
@@ -197,7 +199,11 @@ at_track_build_preview (Track *track)
   }
 
   tmp_region_destroy (region);
+}
 
+void
+at_track_update_preview (Track *track)
+{
   /* left and right previews */
   if (track->preview[0])
     gtk_wave_set_data (GTK_WAVE (track->wave_widget), 0, 
@@ -205,11 +211,8 @@ at_track_build_preview (Track *track)
   if (track->preview[1])
     gtk_wave_set_data (GTK_WAVE (track->wave_widget), 1, 
 		       track->preview[1], track->preview_len);  
-
-  g_print ("done creating track preview\n");
 }
 
-
 void
 at_track_new_cmd_callback (GtkWidget *widget, gpointer data)
 {
@@ -303,6 +306,21 @@ at_track_get_record_track(void)
   return (record_track = tempnam ("./", "atrec"));
 }
 
+/* adjust the preview based on a new volume */
+void
+at_track_adjust_volume_of_preview (Track *track, gfloat old_volume, gfloat new_volume)
+{
+  gfloat diff;
+  gint i;
+
+  diff = new_volume / old_volume;
+
+  for (i=0; i < track->preview_len; i++) {
+    track->preview[0][i] *= diff;
+    track->preview[1][i] *= diff;
+  }
+}
+
 void
 at_track_record_done (Clip *clip)
 {
@@ -346,7 +364,10 @@ at_track_record_done (Clip *clip)
       g_warning ("screwed up record track");
     }
 
+  /* track->preview_dirty = TRUE; */
+  
   at_track_build_preview (track);
+  at_track_update_preview (track);
 }
 
 
@@ -374,38 +395,40 @@ at_track_mute_cmd_callback (GtkWidget *widget, gpointer *data)
     /* this one is muted now, we have to remix. */
     clip = at_clip_get_ID (track->clip_ID);
     clip->dirty = TRUE;
-    at_clip_update(clip);
+    
+    if (!track->is_composite) 
+      at_delay (1000, at_clip_delayed_update, clip);
 }
 
 static void 
-at_track_wave_compression_cmd_callback (GtkWidget *widget, gpointer *data)
+at_track_volume_changed_cmd_callback (GtkWidget *widget, gpointer *data)
 {
     gint track_ID = (gint) data;
     gint range;
+    gfloat volume;
     Track *track;
     GtkWidget *spinner;
-    
+    Clip *clip;
+
     track = at_track_get_ID (track_ID);
     
     spinner = track->spinner;
     
     range = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spinner));
 
-    if (range == track->preview_compression)
-      return;
+    volume = (gfloat) range / 100.0;
+    
+    /* rebuild the preview by adjusting from new volume */
+    at_track_adjust_volume_of_preview (track, track->volume, volume);
+    at_track_update_preview (track);
 
-    track->preview_compression = range * 10;
-
-    at_track_build_preview (track);
-
-    /* left and right previews */
-    if (track->preview[0])
-      gtk_wave_set_data (GTK_WAVE (track->wave_widget), 0, 
-			 track->preview[0], track->preview_len);  
-    if (track->preview[1])
-      gtk_wave_set_data (GTK_WAVE (track->wave_widget), 1, 
-			 track->preview[1], track->preview_len);  
+    track->volume = volume;
+    track->dirty = TRUE;
 
+    clip = at_clip_get_ID (track->clip_ID);
+    
+    if (!track->is_composite) 
+      at_delay (1000, at_clip_delayed_update, clip);
 }
 
 GtkWidget *at_track_gui_new (View *view, gint id)
@@ -449,13 +472,13 @@ GtkWidget *at_track_gui_new (View *view, gint id)
     gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
     
     /* spin button for wave range */
-    adj = (GtkAdjustment *) gtk_adjustment_new ((gfloat) track->preview_compression / 10, 
-						10.0, 200.0, 10.0,
+    adj = (GtkAdjustment *) gtk_adjustment_new (track->volume * 100, 
+						0.0, 800.0, 5.0, 
 						5.0, 0.0);
     spinner = track->spinner = gtk_spin_button_new (adj, 0, 0);
     gtk_box_pack_start (GTK_BOX (vbox), spinner, FALSE, TRUE, 0);
     gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
-			GTK_SIGNAL_FUNC (at_track_wave_compression_cmd_callback),
+			GTK_SIGNAL_FUNC (at_track_volume_changed_cmd_callback),
 			(gpointer) id);
 
     /* wave widget */
@@ -464,12 +487,7 @@ GtkWidget *at_track_gui_new (View *view, gint id)
     gtk_wave_set_vertical_range (GTK_WAVE (track->wave_widget), 2);
     gtk_box_pack_start (GTK_BOX (hbox), track->wave_widget, TRUE, TRUE, 2);
 
-    if (track->preview[0])
-      gtk_wave_set_data (GTK_WAVE (track->wave_widget), 0, 
-			 track->preview[0], track->preview_len);  
-    if (track->preview[1])
-      gtk_wave_set_data (GTK_WAVE (track->wave_widget), 1, 
-			 track->preview[1], track->preview_len);  
+    at_track_update_preview (track);
 
     gtk_object_set_user_data (GTK_OBJECT (main_hbox), (gpointer) id);
 
