@@ -16,12 +16,12 @@ struct _Track
 			       * Currently it must be float */
   gint preview_len;           /* Length of Preview */
   gint preview_compression;   /* track->length / track->preview_compression for preview length */
+  gint preview_dirty;         /* does the preview need rebuilding ? */
 
   gchar *name;                /* name of the track        */
 
   gint mute;                  /* is this track muted?     */
-  gint read_only;             /* basically just for composite.
-				 can't record on that one. */
+  gint is_composite;          /* is this the composite track ? */
 
   GSList *adjustments;        /* track adjustments...     */
 
@@ -49,6 +49,10 @@ Track *          at_track_copy                   (Track *track);
 void             at_track_translate              (Track *track, guint32);
 void             at_track_resize                 (Track *track, int, guint32); 
 void             at_track_build_preview          (Track *track);
+void             at_track_update_preview         (Track *track);
+void             at_track_adjust_volume_of_preview (Track *track, 
+						  gfloat old_volume, 
+						  gfloat new_volume);
 
 
 /* access funtions */
