@@ -173,6 +173,7 @@ tmp_region_new (AudioRegion *src,
 		size_t length)
 {
   TmpRegion *new;
+  static void *last_address = NULL;
 
   g_print ("--> requesting region from file '%s' - ref %d - length %d", 
 	   src->filename, src->tmp_region_ref, length);
@@ -198,8 +199,10 @@ tmp_region_new (AudioRegion *src,
   new->length = length;
 
   new->data = mmap (NULL, length * new->bps, PROT_READ | PROT_WRITE,
-		    MAP_FILE | MAP_SHARED, src->fd, 
+		    MAP_SHARED, src->fd, 
 		    src->offset + (offset * src->bps));
+  
+  last_address = new->data;
 
   g_print (" - size %d\n", length * new->bps);
 
